"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// builtin
const fs_1 = require("fs");
const path_1 = require("path");
const process_1 = require("process");
const nodeVersion = String(process_1.versions.node || '0');
// external
const fs_accessible_1 = require("@bevry/fs-accessible");
const errlop_1 = __importDefault(require("errlop"));
const version_compare_1 = __importDefault(require("version-compare"));
/** Create a directory. */
async function mkdirp(path, opts = {}) {
    if (Array.isArray(path)) {
        return Promise.all(path.map((i) => mkdirp(i, opts))).then(() => { });
    }
    // if we already exist, nothing to do
    if (await (0, fs_accessible_1.isAccessible)(path)) {
        return;
    }
    // create according to node.js version
    if ((0, version_compare_1.default)(nodeVersion, '14.14.0') >= 0) {
        return new Promise(function (resolve, reject) {
            (0, fs_1.mkdir)(path, Object.assign({
                recursive: true,
            }, opts), function (err) {
                if (err) {
                    if (err.code === 'EEXIST') {
                        // something else created it before we could create it
                        return resolve();
                    }
                    return reject(new errlop_1.default(`failed to make the directory: ${path}`, err));
                }
                resolve();
            });
        });
    }
    else {
        const dir = (0, path_1.dirname)(path);
        await mkdirp(dir, opts);
        return new Promise(function (resolve, reject) {
            (0, fs_1.mkdir)(path, Object.assign({}, opts), function (err) {
                if (err) {
                    if (err.code === 'EEXIST') {
                        // something else created it before we could create it
                        return resolve();
                    }
                    return reject(new errlop_1.default(`failed to make the directory: ${path}`, err));
                }
                resolve();
            });
        });
    }
}
exports.default = mkdirp;
